@@ -15,20 +15,22 @@ use XML::LibXML;
 use XML::Simple;
 use Params::Validate qw(:all);
 use IO::All;
-use File::Path;
+use File::Path 'make_path';
 use File::Basename qw'dirname';
 use File::Spec;
 use File::Copy;
 use Rex::Repositorio::Repository_Factory;
 use JSON::XS;
 use Data::Dumper;
-use Term::ProgressBar;
-use Term::ANSIColor;
 
-our $VERSION = '1.1.0'; # VERSION
+our $VERSION = '1.2.0'; # VERSION
 
-has config => ( is => 'ro' );
-has logger => ( is => 'ro' );
+if ( !$Rex::Repositorio::VERSION ) {
+  $Rex::Repositorio::VERSION = "9999.99.99";
+}
+
+has config     => ( is => 'ro' );
+has logger     => ( is => 'ro' );
 
 sub ua {
   my ( $self, %option ) = @_;
@@ -53,7 +55,16 @@ sub ua {
 sub run {
   my ( $self, %option ) = @_;
 
+  # this config checking/munging stuff should probably be in the 'has config' definition?
   $self->config->{RepositoryRoot} =~ s/\/$//;
+  $self->logger->log_and_croak(level => 'error', message => qq/"all" is a reserved word and cannot be used as a repo name\n/)
+    if grep { $_ eq 'all' } keys %{ $self->config->{Repository} };
+  $self->config->{TagStyle} ||= 'TopDir';
+  $self->logger->log_and_croak(
+    level => 'error', message => sprintf "Unknown TagStyle %s, must be TopDir or BottomDir\n", $self->config->{TagStyle}
+  )
+    unless $self->config->{TagStyle} =~ m/^(?:Top|Bottom)Dir$/;
+
   $self->parse_cli_option(%option);
 }
 
@@ -65,10 +76,14 @@ sub parse_cli_option {
     exit 0;
   }
 
+  if ( exists $option{repo} ) {
+    $self->logger->log_and_croak(level => 'error', message => sprintf("Unknown repo: %s\n", $option{repo}))
+      unless $option{repo} eq 'all'
+        or $self->config->{Repository}->{ $option{repo} };
+  }
+
   if ( exists $option{mirror} && exists $option{repo} ) {
-    $self->print_info(
-      "Going to mirror " . $option{repo} . ". This may take a while." );
-    print "\n";
+    $self->logger->info("Going to mirror $option{repo} This may take a while." );
 
     my $update_files = 1;
 
@@ -79,19 +94,22 @@ sub parse_cli_option {
 
     $self->mirror(
       repo            => $option{repo},
+      checksums       => $option{'checksums'},
       update_metadata => ( $option{"update-metadata"} || 0 ),
       update_files    => $update_files,
       force           => ( $option{"force-download"} || 0 ),
     );
 
-    print "\n";
-    print "\n";
-    $self->print_info( "Finished downloading of files for " . $option{repo} );
-    print "\n";
+    $self->logger->info("Finished downloading of files for $option{repo}");
   }
 
   elsif ( exists $option{tag} && exists $option{repo} ) {
-    $self->tag( tag => $option{tag}, repo => $option{repo} );
+    $self->tag(
+      tag => $option{tag},
+      clonetag => $option{clonetag} || 'head',
+      repo => $option{repo},
+      force => $option{force} || 0,
+    );
   }
 
   elsif ( exists $option{repo} && exists $option{"update-errata"} ) {
@@ -350,6 +368,10 @@ sub mirror {
       repo => {
         type => SCALAR
       },
+      checksums => {
+        type     => BOOLEAN,
+        optional => 1,
+      },
       update_metadata => {
         type     => BOOLEAN,
         optional => 1,
@@ -385,6 +407,7 @@ sub mirror {
     );
 
     $repo_o->mirror(
+      checksums       => $option{checksums},
       update_metadata => $option{update_metadata},
       update_files    => $option{update_files},
       force           => $option{force},
@@ -403,6 +426,12 @@ sub tag {
       tag => {
         type => SCALAR
       },
+      clonetag => {
+        type => SCALAR
+      },
+      force => {
+        type => BOOLEAN
+      }
     }
   );
 
@@ -410,31 +439,48 @@ sub tag {
   my $root_dir    = $self->config->{RepositoryRoot};
   $repo_config->{local} =~ s/\/$//;
 
-  my @dirs    = ("$root_dir/head/$repo_config->{local}");
-  my $tag_dir = "$root_dir/$option{tag}/$repo_config->{local}";
+  # Why is this an array?
+  my @dirs;
+  push @dirs, $self->get_repo_dir(repo => $option{repo}, tag => $option{clonetag});
+  my $tag_dir = $self->get_repo_dir(repo => $option{repo}, tag => $option{tag});
 
-  mkpath $tag_dir;
+  $self->logger->log_and_croak(level => 'error', message => "Unknown tag $option{clonetag} on repo $option{repo} ($dirs[0])\n")
+    unless ( -d $dirs[0] );
+
+  if ( -e $tag_dir ) {
+    if( $option{force} ) {
+      $self->logger->debug("Removing $tag_dir");
+      rmtree $tag_dir; # should be remove_tree, but will use legacy to match mkdir
+    }
+    else {
+      $self->logger->log_and_croak(level => 'error', message => "Tag $option{tag} on repo $option{repo} already exists (${tag_dir}), use --force\n");
+    }
+  }
+
+  make_path($tag_dir);
 
   for my $dir (@dirs) {
-    opendir my $dh, $dir;
+    opendir my $dh, $dir
+        or $self->logger->log_and_croak(level => 'error', message => "Failed to open $dir: $!\nNew tag is probably unusable\n");
     while ( my $entry = readdir $dh ) {
-      next if ( $entry eq "." || $entry eq ".." );
-      my $rel_entry = "$dir/$entry";
-      $rel_entry =~ s/$root_dir\/head\/$repo_config->{local}\///;
+      next if ( $entry eq '.' || $entry eq '..' );
+      my $rel_entry = File::Spec->catfile($dir, $entry);
+      $rel_entry =~ s{^$dirs[0]/}{}; # TODO use File::Spec?
 
-      if ( -d "$dir/$entry" ) {
-        push @dirs, "$dir/$entry";
-        $self->logger->debug("Creating directory: $tag_dir/$rel_entry.");
-        mkdir "$tag_dir/$rel_entry";
+      my $srcfile = File::Spec->catfile($dir,$entry);
+      my $dstfile = File::Spec->catfile($tag_dir,$rel_entry);
+      $self->logger->debug("Tag Src: $srcfile, Dst: $dstfile");
+
+      if ( -d $srcfile ) {
+        push @dirs, $srcfile;
+        $self->logger->debug("Creating directory: $dstfile");
+        mkdir $dstfile;
         next;
       }
 
       $self->logger->debug(
-        "Linking (hard): $dir/$entry -> $tag_dir/$rel_entry");
-      if ( -f File::Spec->catfile( $tag_dir, $rel_entry ) ) {
-        unlink File::Spec->catfile( $tag_dir, $rel_entry );
-      }
-      link "$dir/$entry", "$tag_dir/$rel_entry";
+        "Linking (hard): $srcfile -> $dstfile");
+      link $srcfile, $dstfile;
     }
     closedir $dh;
   }
@@ -454,75 +500,57 @@ sub get_errata_dir {
     }
   );
 
-  return File::Spec->catdir(
-    File::Spec->rel2abs( $self->config->{RepositoryRoot} ),
-    $option{tag}, $option{repo}, "errata" );
+  if ($self->config->{TagStyle} eq 'TopDir') {
+    return File::Spec->catdir(
+      File::Spec->rel2abs( $self->config->{RepositoryRoot} ),
+      $option{tag}, $option{repo}, 'errata' );
+  }
+  elsif ($self->config->{TagStyle} eq 'BottomDir') {
+    return File::Spec->catdir(
+      File::Spec->rel2abs( $self->config->{RepositoryRoot} ),
+      $option{repo}, $option{tag}, 'errata' );
+  }
+  else {
+    # add other styles here
+    $self->logger->log_and_croak(level => 'error', message => 'Shouldnt have gotten here');
+  }
+
 }
 
 sub get_repo_dir {
-  my $self   = shift;
+  my $self = shift;
   my %option = validate(
     @_,
     {
       repo => {
         type => SCALAR
-      }
-    }
-  );
-
-  return File::Spec->rel2abs( $self->config->{RepositoryRoot}
-      . "/head/"
-      . $self->config->{Repository}->{ $option{repo} }->{local} );
-}
-
-sub progress_bar {
-  my $self   = shift;
-  my %option = validate(
-    @_,
-    {
-      title => {
-        type => SCALAR,
       },
-      length => {
+      tag => {
         type => SCALAR,
+        default => 'head',
       }
     }
   );
 
-  $self->print_info( $option{title} );
-  print "\n";
+  my $repo_config = $self->config->{Repository}->{ $option{repo} };
 
-  my $pr = Term::ProgressBar->new( { count => $option{length} } );
-  return $pr;
-}
-
-sub print_info {
-  my ( $self, $msg ) = @_;
-  print color "bold green";
-  print ">> ";
-  print color "reset";
-
-  my @parts = split( / /, $msg );
-  my $current_line_len = 3;
-
-  for my $part (@parts) {
-    $current_line_len += length $part;
-    if ( $current_line_len >= 80 ) {
-      print "\n   ";
-      $current_line_len = 3;
-    }
-
-    print "$part ";
+  if ($self->config->{TagStyle} eq 'TopDir') {
+    return File::Spec->catdir($self->config->{RepositoryRoot}, $option{tag}, $repo_config->{local});
+  }
+  elsif ($self->config->{TagStyle} eq 'BottomDir') {
+    return File::Spec->catdir($self->config->{RepositoryRoot}, $repo_config->{local}, $option{tag});
+  }
+  else {
+    $self->logger->log_and_croak(level => 'error', message => 'get_repo_dir: Unknown TagStyle: '.$self->config->{TagStyle});
   }
 
-  print "\n";
 }
 
 sub _print {
   my $self  = shift;
   my @lines = @_;
 
-  print "repositorio: $VERSION\n";
+  print "repositorio: ${Rex::Repositorio::VERSION}\n";
   print "-" x 80;
   print "\n";
   print "$_\n" for @lines;
@@ -532,12 +560,14 @@ sub _help {
   my ($self) = @_;
 
   $self->_print(
-    "--mirror            mirror a configured repository (needs --repo)",
+    "--mirror            mirror a configured repository (needs --repo, use \"all\" for all repos)",
     "--tag=tagname       tag a repository (needs --repo)",
+    "--clonetag=tagname  clones a tag in a repository (needs --repo and new --tag)",
     "--repo=reponame     the name of the repository to use",
     "--update-metadata   update the metadata of a repository",
     "--update-files      download files even if they are already downloaded",
     "--force-download    force the download of already downloaded files",
+    "--checksums         when mirroring verify local packages with checksums instead of size",
     "--no-update-files   do not download packages",
     "--init              initialize an empty repository",
     "--add-file=file     add a file to a repository (needs --repo)",
@@ -549,6 +579,7 @@ sub _help {
     "  --package=pkg     for which package the errata should be queries",
     "  --version=ver     for which version of a package the errata should be queries",
     "  --arch=arch       for which architecture of a package the errata should be queries",
+    "--loglevel          change the stdout log level (debug,info,notice,warning,error,critical)",
     "--help              display this help message",
   );
 
@@ -591,10 +622,12 @@ create consistant installations of your server.
 
 =over 4
 
-=item --mirror            mirror a configured repository (needs --repo)
+=item --mirror            mirror a configured repository (needs --repo, use "all" for all repos)
 
 =item --tag=tagname       tag a repository (needs --repo)
 
+=item --clonetag=tagname  clones a tag in a repository (needs --repo and new --tag)
+
 =item --repo=reponame     the name of the repository to use
 
 =item --update-metadata   update the metadata of a repository
@@ -630,12 +663,14 @@ create consistant installations of your server.
 To configure repositor.io create a configuration file
 I</etc/rex/repositorio.conf>.
  RepositoryRoot = /srv/html/repo/
-   
+
+ TagStyle = TopDir
+
  # log4perl configuration file
  <Log4perl>
    config = /etc/rex/io/log4perl.conf
  </Log4perl>
-   
+
  # create a mirror of the nightly rex repository
  # the files will be stored in
  # /srv/html/repo/head/rex-centos-6-x86-64/CentOS/6/rex/x86_64/
@@ -644,7 +679,7 @@ I</etc/rex/repositorio.conf>.
    local = rex-centos-6-x86-64/CentOS/6/rex/x86_64/
    type  = Yum
  </Repository>
-   
+
  # create a mirror of centos 6
  # and download the pxe boot files, too.
  <Repository centos-6-x86-64>
@@ -653,13 +688,13 @@ I</etc/rex/repositorio.conf>.
    type   = Yum
    images = true
  </Repository>
-   
+
  # create a custom repository
  <Repository centos-6-x86-64-mixed>
    local = centos-6-x86-64-mixed/mixed/6/x86_64/
    type  = Yum
  </Repository>
-   
+
  <Repository debian-wheezy-i386-main>
    url       = http://ftp.de.debian.org/debian/
    local     = debian-wheezy-amd64-main/debian
@@ -668,7 +703,7 @@ I</etc/rex/repositorio.conf>.
    dist      = wheezy
    component = main
  </Repository>
- 
+
 If you want to sign your custom repositories you have to configure the gpg key to use.
 repositorio automatically exports the public key into the root of the repository, so it can be imported from the clients.
 If you don't specify the gpg password repositorio will ask you for the password.
@@ -705,3 +740,11 @@ An example log4perl.conf file:
  log4perl.appender.FileAppndr1          = Log::Log4perl::Appender::File
  log4perl.appender.FileAppndr1.filename = /var/log/repositorio.log
  log4perl.appender.FileAppndr1.layout   = Log::Log4perl::Layout::SimpleLayout
+
+
+=head1 LICENSE
+
+Repositorio is a free software, licensed under:
+The Apache License, Version 2.0, January 2004
+
+
