@@ -14,10 +14,9 @@ use Digest::SHA;
 use Carp;
 use Params::Validate qw(:all);
 use File::Spec;
-use File::Path;
 use IO::All;
 
-our $VERSION = '1.1.0'; # VERSION
+our $VERSION = '1.2.0'; # VERSION
 
 extends "Rex::Repositorio::Repository::Base";
 
@@ -35,54 +34,52 @@ sub mirror {
   ##############################################################################
   # get meta data
   ##############################################################################
-  my $url      = $self->repo->{url} . "/dists/$dist";
+  my $url      = $self->repo->{url} . "/dists/${dist}";
   my $contents = $self->download("$url/Release");
   my $ref      = $self->_parse_debian_release_file($contents);
   my $arch     = $self->repo->{arch};
 
-  my $pr = $self->app->progress_bar(
-    title  => "Downloading metadata...",
-    length => 2,
-  );
+  $self->app->logger->notice('Downloading metadata...');
+
+  my $destbase     = $self->app->get_repo_dir(repo => $self->repo->{name});
 
   # try download Release and Release.gpg
   try {
     $self->download_metadata(
-      url   => $url . "/Release",
-      dest  => $self->repo->{local} . "/dists/$dist/Release",
+      url   => $url . '/Release',
+      dest  => File::Spec->catfile('dists',$dist,'Release'),
       force => $option{update_metadata},
     );
 
-    $pr->update(1);
+    $self->app->logger->info("1/2 ${url}");
 
     $self->download_metadata(
-      url   => $url . "/Release.gpg",
-      dest  => $self->repo->{local} . "/dists/$dist/Release.gpg",
+      url   => $url . '/Release.gpg',
+      dest  => File::Spec->catfile('dists', $dist, 'Release.gpg'),
       force => $option{update_metadata},
     );
 
-    $pr->update(2);
+    $self->app->logger->info("2/2 ${url}");
 
   }
   catch {
-    $pr->update(2);
+    $self->app->logger->info("2/2 ${url}");
     $self->app->logger->error($_);
   };
 
-  my $i = 0;
-  print "\n";
-  print "\n";
-  $pr = $self->app->progress_bar(
-    title  => "Downloading file listing...",
-    length => scalar( @{ $ref->{SHA1} || $ref->{SHA1Sum} } ),
-  );
+
+  $self->app->logger->notice('Downloading file listing...');
+  my $f_count = 0;
+  my $f_total = scalar( @{ $ref->{SHA1} || $ref->{SHA1Sum} } );
 
   for my $file_data ( @{ $ref->{SHA1} || $ref->{SHA1Sum} } ) {
-    $i++;
-    $pr->update($i);
 
     my $file_url = $url . "/" . $file_data->{file};
     my $file     = $file_data->{file};
+
+    $f_count++;
+    $self->app->logger->info("${f_count}/${f_total} ${file_url}");
+
     my $arch_str = join( "|", @archs );
     my $regexp   = qr{i18n|((Contents|binary|installer)\-(udeb-)?($arch_str))};
     next
@@ -91,7 +88,7 @@ sub mirror {
     try {
       $self->download_metadata(
         url   => $file_url,
-        dest  => $self->repo->{local} . "/dists/$dist/$file",
+        dest  => File::Spec->catfile('dists',$dist,$file),
         force => $option{update_metadata},
       );
     }
@@ -116,36 +113,33 @@ sub mirror {
     }
     for my $component (@components) {
 
-      my $local_components_path =
-        $self->app->get_repo_dir( repo => $self->repo->{name} )
-        . "/dists/$dist/$component";
+      my $local_components_path = File::Spec->catdir(
+        $self->app->get_repo_dir( repo => $self->repo->{name} ),
+        'dists',$dist,$component);
 
       for my $arch (@archs) {
         $self->app->logger->debug(
           "Processing ($name, $component) $dist / $arch");
 
-        my $local_packages_path =
-          $local_components_path . "/binary-$arch/Packages.gz";
+        my $local_packages_path = File::Spec->catfile(
+          $local_components_path, "binary-$arch", 'Packages.gz');
 
         $self->app->logger->debug("Reading: $local_packages_path");
         my $content = $self->gunzip( io($local_packages_path)->binary->all );
         my $package_ref = $self->_parse_debian_package_file($content);
 
-        print "\n";
-        print "\n";
-        $pr = $self->app->progress_bar(
-          title  => "Downloading packages for $component ($arch)...",
-          length => scalar( @{$package_ref} ),
-        );
-        my $pi = 0;
+        $self->app->logger->notice("Downloading packages for ${component} (${arch})...");
+        my $p_count = 0;
+        my $p_total = scalar( @{$package_ref} );
 
         for my $package ( @{$package_ref} ) {
-          $pi++;
-          $pr->update($pi);
           my $package_url  = $self->repo->{url} . "/" . $package->{Filename};
           my $package_name = $package->{Package};
 
-          my $local_file = $self->repo->{local} . "/" . $package->{Filename};
+          $p_count++;
+          $self->app->logger->info("${p_count}/$p_total ${package_url}");
+
+          my $local_file = File::Spec->catfile($package->{Filename});
           $self->download_package(
             url  => $package_url,
             name => $package_name,
@@ -200,21 +194,18 @@ sub mirror {
           my $content = $self->gunzip( io($local_packages_path)->binary->all );
           my $package_ref = $self->_parse_debian_package_file($content);
 
-          print "\n";
-          print "\n";
-          $pr = $self->app->progress_bar(
-            title => "Downloading installer packages for $component ($arch)...",
-            length => scalar( @{$package_ref} ),
-          );
-          my $pi = 0;
+          $self->app->logger->notice("Downloading installer packages for ${component} (${arch})...");
+          my $p_count = 0;
+          my $p_total = scalar( @{$package_ref} );
 
           for my $package ( @{$package_ref} ) {
-            $pi++;
-            $pr->update($pi);
             my $package_url  = $self->repo->{url} . "/" . $package->{Filename};
             my $package_name = $package->{Package};
 
-            my $local_file = $self->repo->{local} . "/" . $package->{Filename};
+            $p_count++;
+            $self->app->logger->info("${p_count}/${p_total} ${package_url}");
+
+            my $local_file = File::Spec->catfile($package->{Filename});
             $self->download_package(
               url  => $package_url,
               name => $package_name,
@@ -232,8 +223,8 @@ sub mirror {
             "current", "images" );
 
           $self->app->logger->debug( "Looking for SHA256SUMS file: "
-              . File::Spec->catfile( $local_file_path, "/SHA256SUMS" ) );
-          if ( !-f File::Spec->catfile( $local_file_path, "/SHA256SUMS" ) ) {
+              . File::Spec->catfile( $local_file_path, "SHA256SUMS" ) );
+          if ( !-f File::Spec->catfile( $local_file_path, "SHA256SUMS" ) ) {
             $self->app->logger->error(
               "need to download SHA256SUMS file, because it was not listed in Release file"
             );
@@ -243,8 +234,8 @@ sub mirror {
             $self->app->logger->debug("Found repo root: $repo_root");
             $remote_sha256sums =~ s/^\Q$repo_root\E//;
 
-            my $local_sha256sums_rel =
-              $self->repo->{local} . "/" . $remote_sha256sums;
+            my $local_sha256sums_rel = File::Spec->catfile(
+              $remote_sha256sums);
 
             $remote_sha256sums = $self->repo->{url} . $remote_sha256sums;
 
@@ -259,26 +250,22 @@ sub mirror {
           }
 
           my $file_ref =
-            $self->_parse_sha256sum_file( $local_file_path . "/SHA256SUMS" );
+            $self->_parse_sha256sum_file( File::Spec->catfile($local_file_path, 'SHA256SUMS' ));
 
-          print "\n";
-          print "\n";
-          $pr = $self->app->progress_bar(
-            title =>
-              "Downloading installer image files for $component ($arch)...",
-            length => scalar( @{$package_ref} ),
-          );
-          my $fi = 0;
+          $self->app->logger->notice("Downloading installer image files for ${component} (${arch})...");
+          my $f_count = 0;
+          my $f_total = scalar( @{$package_ref} );
 
           for my $file ( @{$file_ref} ) {
-            $fi++;
-            $pr->update($fi);
             my $file_url =
                 $self->repo->{url}
               . "/dists/$dist/$component/installer-$arch/current/images/"
               . $file->{file};
             my $file_name = $file->{file};
 
+            $f_count++;
+            $self->app->logger->info("${f_count}/$f_total ${file_url}");
+
             my $local_file = File::Spec->catfile( $self->repo->{local},
               "dists", $dist, $component, "installer-$arch", "current",
               "images", $file->{file} );
@@ -300,25 +287,22 @@ sub mirror {
   # download rest of metadata
   ##############################################################################
 
-  print "\n";
-  print "\n";
-  $pr = $self->app->progress_bar(
-    title  => "Downloading rest of metadata...",
-    length => ( 2 * scalar(@archs) ),
-  );
+  $self->app->logger->notice("Downloading rest of metadata...");
+  my $m_count = 0;
+  my $m_total = ( 2 * scalar(@archs) );
 
-  my $mi = 0;
   for my $arch (@archs) {
     for my $suffix (qw/bz2 gz/) {
-      $mi++;
-      $pr->update($mi);
       my $file_url = $url . "/Contents-$arch.$suffix";
       my $file     = "Contents-$arch.$suffix";
 
+      $m_count++;
+      $self->app->logger->info("${m_count}/$m_total ${file_url}");
+
       try {
         $self->download_metadata(
           url   => $file_url,
-          dest  => $self->repo->{local} . "/dists/$dist/$file",
+          dest  => File::Spec->catfile('dists',$dist,$file),
           force => $option{update_metadata},
         );
       }
@@ -430,12 +414,9 @@ sub init {
   my $desc      = $self->repo->{description} || "$component repository";
 
   my $repo_dir = $self->app->get_repo_dir( repo => $self->repo->{name} );
-  mkpath "$repo_dir/dists/$dist/$component/binary-$arch";
 
-  my $pool_dir = $self->app->get_repo_dir( repo => $self->repo->{name} ) . "/"
-    . "pool/$dist/$component/";
-
-  mkpath $pool_dir;
+  File::Path->make_path(File::Spec->catdir($repo_dir,'dists',$dist,$component,"binary-$arch"));
+  File::Path->make_path(File::Spec->catdir($repo_dir,'pool',$dist,$component));
 
   my $aptftp      = io("$repo_dir/aptftp.conf");
   my $aptgenerate = io("$repo_dir/aptgenerate.conf");
@@ -491,10 +472,9 @@ sub add_file {
   my $dist      = $self->repo->{dist};
   my $component = $self->repo->{component};
 
-  my $dest =
-      $self->app->get_repo_dir( repo => $self->repo->{name} ) . "/"
-    . "pool/$dist/$component/"
-    . basename( $option{file} );
+  my $dest = File::Spec->catdir(
+      $self->app->get_repo_dir( repo => $self->repo->{name} ),
+     'pool',$dist,$component,basename( $option{file} ));
 
   $self->add_file_to_repo( source => $option{file}, dest => $dest );
 
@@ -516,10 +496,9 @@ sub remove_file {
   my $dist      = $self->repo->{dist};
   my $component = $self->repo->{component};
 
-  my $file =
-      $self->app->get_repo_dir( repo => $self->repo->{name} ) . "/"
-    . "pool/$dist/$component/"
-    . basename( $option{file} );
+  my $file = File::Spec->catdir(
+      $self->app->get_repo_dir( repo => $self->repo->{name} ),
+     'pool',$dist,$component,basename( $option{file} ));
 
   $self->remove_file_from_repo( file => $file );
 
@@ -532,6 +511,8 @@ sub _run_ftp_archive {
   my $dist = $self->repo->{dist};
   my $repo_dir = $self->app->get_repo_dir( repo => $self->repo->{name} );
 
+  # TODO: should probably check that apt-ftparchive exists and is executable
+
   system
     "cd $repo_dir ; apt-ftparchive generate -c=aptftp.conf aptgenerate.conf";
 
