@@ -16,17 +16,16 @@ use XML::LibXML;
 use XML::Simple;
 use Params::Validate qw(:all);
 use IO::All;
-use File::Path;
+use File::Path 'make_path';
 use File::Basename qw'dirname';
 use File::Spec;
 use File::Copy;
 use Digest::SHA;
 use Digest::MD5;
-use Term::ReadKey;
 use JSON::XS;
 use List::MoreUtils 'firstidx';
 
-our $VERSION = '1.1.0'; # VERSION
+our $VERSION = '1.2.0'; # VERSION
 
 has app  => ( is => 'ro' );
 has repo => ( is => 'ro' );
@@ -55,13 +54,12 @@ sub download_gzip {
 
   require Compress::Zlib;
 
-  $self->app->logger->debug("Starting uncompressing of: $url");
-
+  my $t1 = time();
   my $un_content = Compress::Zlib::memGunzip($content);
-  $self->app->logger->debug("Finished uncompressing of: $url");
+  my $tdiff = time() - $t1;
+  $self->app->logger->debug("Uncompressing: $url took: ${tdiff} seconds");
   if ( !$un_content ) {
-    $self->app->logger->error("Error uncompressing data.");
-    confess "Error uncompressing data.";
+    $self->app->logger->log_and_croak(level => 'error', message => 'Error uncompressing data.');
   }
 
   return $un_content;
@@ -77,27 +75,34 @@ sub gunzip {
 sub download {
   my ( $self, $url ) = @_;
 
-  $self->app->logger->debug("Starting download of: $url");
-  my $resp = $self->ua->get($url);
-  $self->app->logger->debug("Finished download of: $url");
+  my $retry_count = 0;
+  my $max_retries = $self->app->config->{DownloadRetryCount} // 3;
+  my $success;
+  my $content;
+
+  while (!$success && $retry_count <= $max_retries ) {
+    my $t1 = time();
+    my $resp = $self->ua->get($url);
+    my $tdiff = time() - $t1;
+    $self->app->logger->debug("Download: ${url} took: ${tdiff} seconds");
 
-  if ( !$resp->is_success ) {
-    $self->app->logger->error("Can't download $url.");
-    $self->app->logger->error( "Status: " . $resp->status_line );
-    my $retry_count = 1;
-    while ( !$resp->is_success
-      && $retry_count <= ( $self->app->config->{DownloadRetryCount} // 3 ) )
-    {
-      $self->app->logger->error("Retry downloading of url: $url");
-      $retry_count += 1;
-      $resp = $self->ua->get($url);
-    }
     if ( !$resp->is_success ) {
-      confess "Error downloading $url.";
+      $self->app->logger->error("Download: ${url} failed with status: " . $resp->status_line);
+      $retry_count++;
+      if ($retry_count <= $max_retries) {
+        $self->app->logger->error("Download: ${url} retrying");
+      }
+      else {
+        $self->app->logger->log_and_croak(level => 'error', message=> "download: ${url} failed and exhausted all retries.");
+      }
+    }
+    else {
+      $success = 1;
+      $content = $resp->content;
     }
   }
 
-  return $resp->content;
+  return $content;
 }
 
 sub get_xml {
@@ -139,8 +144,9 @@ sub download_package {
     }
   );
 
-  my $package_file =
-    $self->app->config->{RepositoryRoot} . "/head/" . $option{dest};
+  my $repo_dir = $self->app->get_repo_dir(repo => $self->repo->{name});
+  my $package_file = File::Spec->catfile($repo_dir, $option{dest});
+
   $self->_download_binary_file(
     dest        => $package_file,
     url         => $option{url},
@@ -172,8 +178,9 @@ sub download_metadata {
     }
   );
 
-  my $metadata_file =
-    $self->app->config->{RepositoryRoot} . "/head/" . $option{dest};
+  my $repo_dir = $self->app->get_repo_dir(repo => $self->repo->{name});
+  my $metadata_file = File::Spec->catfile($repo_dir, $option{dest});
+
   $self->_download_binary_file(
     dest  => $metadata_file,
     url   => $option{url},
@@ -208,9 +215,9 @@ sub _download_binary_file {
     }
   );
 
-  $self->app->logger->debug("Downloading: $option{url} -> $option{dest}");
+  $self->app->logger->debug("_download_binary_file: $option{url} -> $option{dest}");
 
-  mkpath( dirname( $option{dest} ) ) if ( !-d dirname $option{dest} );
+  make_path( dirname( $option{dest} ) ) if ( !-d dirname $option{dest} );
 
   if ( exists $option{cb}
     && ref $option{cb} eq "CODE"
@@ -225,44 +232,67 @@ sub _download_binary_file {
       # if callback is failing, we need to download the file once again.
       # so just set force to true
       $self->app->logger->debug(
-        "Setting option force -> 1: update_file is enabled and callback failed."
+        "_download_binary_file: $option{dest} Setting option force -> 1: update_file is enabled and callback failed."
       );
       $option{force} = 1;
     };
   }
 
   if ( -f $option{dest} && !$option{force} ) {
-    $self->app->logger->debug("Skipping $option{url}. File already exists.");
+    $self->app->logger->debug("_download_binary_file: Skipping $option{dest}. File already exists and is the correct checksum");
     return;
   }
 
   if ( !-w dirname( $option{dest} ) ) {
-    $self->app->logger->error( "Can't write to " . dirname( $option{dest} ) );
-    confess "Can't write to " . dirname( $option{dest} );
+    $self->app->logger->log_and_croak( level => 'error', message => "_download_binary_file: Can't write to " . dirname( $option{dest} ) );
   }
 
   if ( -f $option{dest} && $option{force} ) {
+    $self->app->logger->debug("_download_binary_file: $option{dest} force enabled, unlinking");
     unlink $option{dest};
   }
 
-  open my $fh, ">", $option{dest};
-  binmode $fh;
-  my $resp = $self->ua->get(
-    $option{url},
-    ':content_cb' => sub {
-      my ( $data, $response, $protocol ) = @_;
-      print $fh $data;
+  my $retry_count = 0;
+  my $max_retries = $self->app->config->{DownloadRetryCount} // 3;
+  my $success;
+
+  while (!$success && $retry_count <= $max_retries ) {
+    my $t1 = time();
+    open my $fh, '>', $option{dest};
+    binmode $fh;
+    my $resp = $self->ua->get(
+      $option{url},
+      ':content_cb' => sub {
+        my ( $data, $response, $protocol ) = @_;
+        print $fh $data;
+      }
+    );
+    close $fh;
+    my $tdiff = time() - $t1;
+    $self->app->logger->debug("_download_binary_file: $option{url} took: ${tdiff} seconds");
+
+    if ( !$resp->is_success ) {
+      unlink $option{dest};
+      $self->app->logger->error("_download_binary_file: $option{url} failed with status: " . $resp->status_line);
+      $retry_count++;
+      if ($retry_count <= $max_retries) {
+        $self->app->logger->error("_download_binary_file: $option{url} retrying");
+      }
+      else {
+        if($self->app->config->{DownloadSkip404} && $resp->code == 404) {
+          $self->app->logger->error("_download_binary_file: $option{url} failed with status: " . $resp->status_line . ". Ignoring due to config option DownloadSkip404.");
+          return;
+        }
+        else {
+          $self->app->logger->log_and_croak(level => 'error', message=> "_download_binary_file: $option{url} failed and exhausted all retries.");
+        }
+      }
+    }
+    else {
+      $success = 1;
+      $self->app->logger->notice("Downloaded new file: $option{url}");
     }
-  );
-  close $fh;
-
-  if ( !$resp->is_success ) {
-    $self->app->logger->error("Can't download $option{url}.");
-    $self->app->logger->error( "Status: " . $resp->status_line );
-    unlink $option{dest};
-    confess "Error downloading $option{url}.";
   }
-
   $option{cb}->( $option{dest} )
     if ( exists $option{cb} && ref $option{cb} eq "CODE" );
 }
@@ -273,7 +303,12 @@ sub add_file_to_repo {
     @_,
     {
       source => {
-        type => SCALAR
+        type => SCALAR,
+        callbacks => {
+          valid_file => sub {
+            return -f $_[0] ? 1 : 0;
+          },
+        },
       },
       dest => {
         type => SCALAR
@@ -282,16 +317,13 @@ sub add_file_to_repo {
   );
 
   if ( !-f $option{source} ) {
-    $self->app->logger->error("Fild $option{source} not found.");
-    confess "Fild $option{source} not found.";
+    $self->app->logger->log_and_croak(level => 'error', message => "add_file_to_repo: File $option{source} not found.");
   }
 
-  $self->app->logger->debug("Copy $option{source} -> $option{dest}");
+  $self->app->logger->debug("add_file_to_repo: Copy $option{source} -> $option{dest}");
   my $ret = copy $option{source}, $option{dest};
   if ( !$ret ) {
-    $self->app->logger->error(
-      "Error copying file $option{source} to $option{dest}");
-    confess "Error copying file $option{source} to $option{dest}";
+    $self->app->logger->log_and_croak(level => 'error', message => "add_file_to_repo: Error copying file $option{source} to $option{dest}");
   }
 }
 
@@ -330,12 +362,35 @@ sub _checksum_md5 {
 
   close $fh;
 
-  $self->app->logger->debug(
-    "wanted_checksum: $wanted_checksum == $file_checksum");
+  $self->app->logger->debug("_checksum_md5: file: ${file} wanted_checksum: ${wanted_checksum} == ${file_checksum}");
 
   if ( $wanted_checksum ne $file_checksum ) {
-    $self->app->logger->error("Checksum for $file wrong.");
-    confess "Checksum of $file wrong.";
+    $self->app->logger->log_and_croak(level => 'error', message => "_checksum_md5: Checksum for ${file} wrong.");
+  }
+}
+sub _checksum_size {
+  my ( $self, $file, $wanted_size ) = @_;
+
+  my @stats = stat($file);
+  my $file_size = $stats[7];
+
+  $self->app->logger->debug("_checksum_size: file: ${file} wanted_size: ${wanted_size} == ${file_size}");
+
+  if ( $wanted_size ne $file_size ) {
+    $self->app->logger->log_and_croak(level => 'error', message => "_checksum_size: File size for ${file} wrong.");
+  }
+}
+sub _checksum_sha {
+  my ($self, $c_type, $file, $wanted_checksum) = @_;
+
+  my $sha = Digest::SHA->new($c_type);
+  $sha->addfile($file);
+  my $file_checksum = $sha->hexdigest;
+
+  $self->app->logger->debug("_checksum: file: ${file} wanted_checksum: ${wanted_checksum} == ${file_checksum}");
+
+  if ( $wanted_checksum ne $file_checksum ) {
+    $self->app->logger->log_and_croak(level => 'error', message => "_checksum_sha: Checksum for ${file} wrong.");
   }
 }
 
@@ -345,21 +400,16 @@ sub _checksum {
   my $c_type = 1;
   if ( $type eq "sha256" ) {
     $c_type = "256";
+    return $self->_checksum_sha( $c_type, $file, $wanted_checksum );
   }
   elsif ( $type eq "md5" ) {
     return $self->_checksum_md5( $file, $wanted_checksum );
   }
-
-  my $sha = Digest::SHA->new($c_type);
-  $sha->addfile($file);
-  my $file_checksum = $sha->hexdigest;
-
-  $self->app->logger->debug(
-    "wanted_checksum: $wanted_checksum == $file_checksum");
-
-  if ( $wanted_checksum ne $file_checksum ) {
-    $self->app->logger->error("Checksum for $file wrong.");
-    confess "Checksum of $file wrong.";
+  elsif ( $type eq 'size' ) {
+    return $self->_checksum_size( $file, $wanted_checksum );
+  }
+  else {
+    return $self->_checksum_sha( $c_type, $file, $wanted_checksum );
   }
 }
 
@@ -378,10 +428,8 @@ sub read_password {
   $msg ||= "Password: ";
 
   print $msg;
-  ReadMode "noecho";
   my $password = <STDIN>;
   chomp $password;
-  ReadMode 0;
   print "\n";
   return $password;
 }
@@ -461,7 +509,9 @@ sub update_errata {
   $self->app->logger->debug("Updating errata of type: $errata_type");
 
   my $data = $self->download("http://errata.repositor.io/$errata_type.tar.gz");
-  open( my $fh, ">", "/tmp/$errata_type.tar.gz" ) or confess($!);
+  # TODO: use File::Temp
+  my $file = File::Spec->catfile(File::Spec->tmpdir(),"$errata_type.tar.gz");
+  open( my $fh, '>', $file ) or confess($!);
   binmode $fh;
   print $fh $data;
   close($fh);
@@ -469,17 +519,18 @@ sub update_errata {
   my $errata_dir =
     $self->app->get_errata_dir( repo => $self->repo->{name}, tag => "head" );
 
-  mkpath $errata_dir;
+  make_path($errata_dir);
 
-  system "cd $errata_dir ; tar xzf /tmp/$errata_type.tar.gz";
+  #XXX ewww
+  system "cd ${errata_dir} ; tar xzf ${file}"; #TODO: replace with perl
 
   if ( $? != 0 ) {
     confess "Error extracting errata database.";
   }
 
-  unlink "/tmp/$errata_type.tar.gz";
+  unlink $file;
 
-  $self->app->logger->debug("Updating errata of type: $errata_type (done)");
+  $self->app->logger->debug("Updating errata of type: ${errata_type} (done)");
 }
 
 1;
