@@ -13,9 +13,9 @@ use Data::Dumper;
 use Carp;
 use Params::Validate qw(:all);
 use File::Spec;
-use File::Path;
+use File::Path 'make_path';
 
-our $VERSION = '1.1.0'; # VERSION
+our $VERSION = '1.2.0'; # VERSION
 
 extends "Rex::Repositorio::Repository::Base";
 
@@ -23,7 +23,7 @@ sub init {
   my $self = shift;
 
   my $repo_dir = $self->app->get_repo_dir( repo => $self->repo->{name} );
-  mkpath "$repo_dir";
+  make_path "$repo_dir";
 }
 
 1;
