@@ -13,59 +13,53 @@ use Data::Dumper;
 use Carp;
 use Params::Validate qw(:all);
 use File::Spec;
-use File::Path;
+use File::Path 'make_path';
 use IO::All;
 use JSON::XS;
 use Mojo::DOM;
 
-our $VERSION = '1.1.0'; # VERSION
+our $VERSION = '1.2.0'; # VERSION
 
 extends "Rex::Repositorio::Repository::Base";
 
 sub mirror {
   my ( $self, %option ) = @_;
 
-  $self->app->print_info("Collecting files. This may take a while...");
+  $self->app->logger->info('Collecting files. This may take a while...');
 
   my @dirs = ( $self->repo->{url} );
   my @files;
 
   for my $dir (@dirs) {
-    $dir = "$dir/" if ( $dir !~ m/\/$/ );
+    $dir = "$dir/" if ( $dir !~ m{/$} );
     my $content = $self->download($dir);
 
     #my $dom     = Mojo::DOM->new($content);
     my @links = ( $content =~ m/<a[^>+]href=["']?([^"'>]+)["'>]/ig );
 
-    $self->app->print_info("Following $dir...");
-    $self->app->logger->debug("Following $dir");
+    $self->app->logger->debug("Following ${dir}...");
 
-    push @dirs, map { $_ = "$dir$_" }
-      grep { $_ =~ m/\/$/ }
-      grep { $_ !~ m/^\./ } @links;
-    push @files, map { $_ = "$dir$_" }
-      grep { $_ !~ m/\/$/ } @links;
+    push @dirs, map { "${dir}${_}" }
+      grep { $_ =~ m{/$} }
+      grep { $_ !~ m{^\.} } @links;
+    push @files, map { "${dir}${_}" }
+      grep { $_ !~ m{/$} } @links;
   }
 
-  my $pr = $self->app->progress_bar(
-    title  => "Downloading packages...",
-    length => scalar(@files),
-  );
-
-  my $i = 0;
+  my $p_count = 0;
+  my $p_total = scalar(@files);
+  $self->app->logger->info('Downloading packages...');
   for my $file (@files) {
-    $i++;
-    $pr->update($i);
 
     my $path     = $file;
     my $repo_url = $self->repo->{url};
-    $path =~ s/$repo_url//g;
-    my $local_path = File::Spec->catdir(
-      $self->app->get_repo_dir( repo => $self->repo->{name} ),
-      dirname($path) );
-    mkpath $local_path;
+    $p_count++;
+    $self->app->logger->info("${p_count}/${p_total} ${path}");
 
-    my $local_file = $self->repo->{local} . "/" . $path;
+    $path =~ s/$repo_url//g;
+    my $local_file = File::Spec->catdir(
+      $self->app->get_repo_dir(repo => $self->repo->{name}), $path );
+    make_path(basename($local_file));
 
     $self->download_package(
       url  => $file,
@@ -82,8 +76,8 @@ sub mirror {
 sub init {
   my $self = shift;
 
-  my $repo_dir = $self->app->get_repo_dir( repo => $self->repo->{name} );
-  mkpath $repo_dir;
+  my $repo_dir = $self->app->get_repo_dir(repo => $self->repo->{name});
+  make_path($repo_dir);
 }
 
 sub add_file {
@@ -97,8 +91,7 @@ sub add_file {
     }
   );
 
-  my $dest = $self->app->get_repo_dir( repo => $self->repo->{name} ) . "/"
-    . basename( $option{file} );
+  my $dest = File::Spec->catdir($self->app->get_repo_dir(repo => $self->repo->{name}), basename( $option{file} ));
 
   $self->add_file_to_repo( source => $option{file}, dest => $dest );
 }
@@ -115,8 +108,7 @@ sub remove_file {
     }
   );
 
-  my $file = $self->app->get_repo_dir( repo => $self->repo->{name} ) . "/"
-    . basename( $option{file} );
+  my $file = File::Spec->catdir($self->app->get_repo_dir(repo => $self->repo->{name}), $option{file} );
 
   $self->remove_file_from_repo( file => $file );
 }
