@@ -11,7 +11,7 @@ use Params::Validate qw(:all);
 use English;
 use Carp;
 
-our $VERSION = '1.1.0'; # VERSION
+our $VERSION = '1.2.0'; # VERSION
 
 sub create {
   my $class  = shift;
@@ -29,7 +29,7 @@ sub create {
 
   my $type     = $option{type};
   my $repo_mod = "Rex::Repositorio::Repository::$type";
-  eval "use $repo_mod;";
+  eval "use $repo_mod;1;";
   if ($EVAL_ERROR) {
     confess "Error loading repository type: $type. ($EVAL_ERROR)";
   }
