@@ -9,12 +9,12 @@ package Rex::Repositorio::Server::Docker::Helper::Auth;
 use strict;
 use warnings;
 
-our $VERSION = '1.1.0'; # VERSION
+our $VERSION = '1.2.0'; # VERSION
 
 sub create {
   my ( $class, $type, $config ) = @_;
   my $auth_class = "Rex::Repositorio::Server::Docker::Helper::Auth::$type";
-  eval "use $auth_class;";
+  eval "use $auth_class;1;";
   if ($@) {
     die "Error finding auth class: $auth_class.";
   }
