@@ -9,13 +9,13 @@ package Rex::Repositorio::Server::Docker::Image;
 use Mojo::Base 'Mojolicious::Controller';
 use Data::Dumper;
 use File::Spec;
-use File::Path;
+use File::Path 'make_path', 'remove_tree';
 use File::Basename 'basename';
 require IO::All;
 use JSON::XS;
 use MIME::Base64;
 
-our $VERSION = '1.1.0'; # VERSION
+our $VERSION = '1.2.0'; # VERSION
 
 sub put_image {
   my ($self) = @_;
@@ -25,7 +25,7 @@ sub put_image {
     File::Spec->catdir( $repo_dir, "images", $self->param("name") );
   my $image_json = File::Spec->catfile( $image_dir, "image.json" );
 
-  mkpath $image_dir;
+  make_path $image_dir;
 
   open( my $fh, ">", $image_json ) or die($!);
   print $fh $self->req->body;
