@@ -3,16 +3,18 @@
 #
 # vim: set ts=2 sw=2 tw=0:
 # vim: set expandtab:
+#
+use strict;
 
 package Rex::Repositorio::Server::Yum::File;
 
 use Mojo::Base 'Mojolicious::Controller';
 use File::Spec;
-use File::Path;
+use File::Path 'make_path';
 use File::Basename qw'dirname';
 use Mojo::UserAgent;
 
-our $VERSION = '1.1.0'; # VERSION
+our $VERSION = '1.2.0'; # VERSION
 
 sub serve {
   my ($self) = @_;
@@ -139,17 +141,19 @@ sub _proxy_url {
     sub {
       my ( $c, $tx ) = @_;
       $c->app->log->debug("Got data from upstream ($orig_url)...");
-      mkpath( dirname($serve_dir) );
+      make_path( dirname($serve_dir) );
       open my $fh, '>', $serve_dir or die($!);
       binmode $fh;
       print $fh $tx->res->body;
       close $fh;
 
-      my $etag = $tx->res->headers->header('ETag');
-      $etag =~ s/"//g;
-      open my $fh_e, '>', "$serve_dir.etag" or die($!);
-      print $fh_e $etag;
-      close $fh_e;
+      my $etag = $tx->res->headers->header('ETag') || '';
+      if($etag) {
+        $etag =~ s/"//g;
+        open my $fh_e, '>', "$serve_dir.etag" or die($!);
+        print $fh_e $etag;
+        close $fh_e;
+      }
     }
   );
 }
